
HealPoints = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceHook-2.1", "AceEvent-2.0", "AceDB-2.0");

HealPoints.ISHEALER = nil;
local lastLink = nil;
local diff1  = nil;
local diff2 = nil;
local diff3 = nil;
local diff4 = nil;

HealPoints:RegisterDB("HEALPOINTS_CONFIG", "HEALPOINTS_CONFIG");
HealPoints:RegisterDefaults('char', {
    power = {
      spell = "",
      auto = true,
      duration = 1,
      rank = 1,
      mana = 100,
    },
    endurance = {
      spell = "",
      auto = true,
      duration = 5,
      rank = 1,
      mana = 0,
    },
    hot = {
      numtargets = 3,
    },
    tooltips = 1,
    debug = 0,
    gems = {
      blueGem = 1,
      redGem = 1,
      yellowGem = 1,
    }
  });
HealPoints:RegisterChatCommand({ "/healpoints" }, {
  type = "group",
  args = {
    bscan = {
      type = "execute",
      name = "Scan bonuses",
      desc = "List special set/equip bonuses detected.",
      func = "listScannedBonuses"
    },
    config = {
      type = "execute",
      name = "Config",
      desc = "Open the configuration window.",
      func = function()
        if (HealPoints.ISHEALER and HealPoints:IsActive()) then
  			  HealPoints_ConfigFrame:Show();
  			end
      end
    },
    calc = {
      type = "execute",
      name = "Calculator",
      desc = "Open the calculator window.",
      func = function()
        if (HealPoints.ISHEALER and HealPoints:IsActive()) then
  			  HealPoints_CalcFrame:Show();
  			end
      end
    },
    tooltips = {
      type = "toggle",
      name = "Tooltip display",
      desc = "Toggles the display of HealPoints change in item tooltips.",
      get = function()
        return HealPoints.db.char.tooltips == 1;
      end,
      set = function()
        HealPoints.db.char.tooltips = 1 - HealPoints.db.char.tooltips;
      end
    },
    debug = {
      type = "toggle",
      name = "Debug mode",
      desc = "Toggles the display of debug messages.",
      get = function()
        return HealPoints.db.char.debug == 1;
      end,
      set = function()
        HealPoints:clearLinkCache();
        HealPoints.db.char.debug = 1 - HealPoints.db.char.debug;
      end
    },
  },
});

function HealPoints:OnInitialize()
	local _, className = UnitClass("player");
	if (className == "PALADIN" or className == "PRIEST" or className == "DRUID" or className == "SHAMAN") then
		HealPoints.ISHEALER = 1;
    HealPointsSpells:init();
    HealPointsConfigUI:init();
  else
    return;
	end

  self:SecureHook(GameTooltip, "SetBagItem", function(this, bag, slot)
			self:DrawTooltip(GameTooltip, GetContainerItemLink(bag, slot));
    end
  );

  self:SecureHook(GameTooltip, "SetLootItem", function(this, slot)
			self:DrawTooltip(GameTooltip, GetLootSlotLink(slot));
    end
  );

  self:SecureHook(GameTooltip, "SetQuestItem", function(this, unit, slot)
			self:DrawTooltip(GameTooltip, GetQuestItemLink(unit, slot));
    end
  );

  self:SecureHook(GameTooltip, "SetQuestLogItem", function(this, sOpt, slot)
			self:DrawTooltip(GameTooltip, GetQuestLogItemLink(sOpt, slot));
    end
  );

  self:SecureHook(GameTooltip, "SetTradeSkillItem", function(this, skill, slot)
			local link = (slot) and GetTradeSkillReagentItemLink(skill, slot) or GetTradeSkillItemLink(skill);
			self:DrawTooltip(GameTooltip, link);
    end
  );

  self:SecureHook(GameTooltip, "SetMerchantItem", function(this, slot)
			self:DrawTooltip(GameTooltip, GetMerchantItemLink(slot));
    end
  );

  self:SecureHook(GameTooltip, "SetAuctionItem", function(this, unit, slot)
			self:DrawTooltip(GameTooltip, GetAuctionItemLink(unit, slot));
    end
  );

  self:SecureHook(GameTooltip, "SetLootRollItem", function(this, id)
			self:DrawTooltip(GameTooltip, GetLootRollItemLink(id));
    end
  );

  self:SecureHook(GameTooltip, "SetHyperlink", function(this, link)
      self:DrawTooltip(GameTooltip, link)
    end
  );
  
  self:Hook(GameTooltip, "SetInventoryItem", function(this, unit, slot)
			local sItem, sCooldown, sRepair = self.hooks[GameTooltip]["SetInventoryItem"](this, unit, slot);
			if(not sItem) then return nil; end
			if (slot > 39) then -- bank
				self:DrawTooltip(GameTooltip, GetInventoryItemLink(unit, slot));
			end
			return sItem, sCooldown, sRepair;
    end
  , true);

  self:SecureHook("SetItemRef", function(link, name, button)
			if(link and name and ItemRefTooltip) then 
        if( strsub(link, 1, 6) ~= "Player" ) then
          if( ItemRefTooltip:IsVisible()) then
            if(not DressUpFrame:IsVisible()) then
              self:DrawTooltip(ItemRefTooltip, link);
            end
            ItemRefTooltip.isDisplayDone = nil;
          end
        end
      end
    end
  );

  self:SecureHook(HealPointsBS, "update", HealPoints.GearChanged);

  -- Remove old settings
  HEALPOINTS_CONFIG['endurance'] = nil;
  HEALPOINTS_CONFIG['power'] = nil;
  HEALPOINTS_CONFIG['tooltips'] = nil;
  HEALPOINTS_CONFIG['regen'] = nil;
  HEALPOINTS_CONFIG['hot'] = nil;
end

function HealPoints:OnEnable(first)
  if (HealPoints.ISHEALER == 1) then
    self:RegisterEvent("UNIT_AURA", "StatsChanged");
    self:RegisterEvent("UNIT_LEVEL", "StatsChanged");
    self:RegisterEvent("UNIT_MAXMANA", "StatsChanged");
    self:RegisterEvent("UNIT_STATS", "StatsChanged");
    self:RegisterEvent("LEARNED_SPELL_IN_TAB", "SpellsChanged");
    self:RegisterEvent("CHARACTER_POINTS_CHANGED", "TalentsChanged");    
    if (not self:IsHooked(HealPointsBS, "update")) then
      self:SecureHook(HealPointsBS, "update", HealPoints.GearChanged);    
    end
    HealPointsConfigUI:spellsChanged();
    HealPoints_CharFrame:Show();
    if (not first) then
      HealPointsUtil:talentsChanged();
      HealPointsCalculator:updateHealPoints();
    end
  else
    DisableAddOn("HealPoints");
  end
end

function HealPoints:OnDisable()
  HealPoints_CharFrame:Hide();
  HealPoints_CalcFrame:Hide();
  HealPoints_ConfigFrame:Hide();
end

function HealPoints:StatsChanged(target) -- UNIT_AURA, UNIT_LEVEL, UNIT_MAXMANA, UNIT_STATS
  lastLink = nil;
  if (UnitIsUnit(target, "player")) then
    if (HealPoints_CalcFrame:IsVisible()) then
      HealPointsCalculator:updateStats();
      HealPointsCalculatorUI:updateSelectedSpellStats();
    end
    if (HealPoints_CalcFrame:IsVisible() or CharacterFrame:IsVisible()) then
      HealPointsCalculator:updateHealPoints();
    end
  end
end

function HealPoints:SpellsChanged(spelltabnr) -- LEARNED_SPELL_IN_TAB
  lastLink = nil;
  HealPointsSpells:spellsChanged();
  HealPointsCalculatorUI:updateSpellLists();
  HealPointsCalculatorUI:updateSpellTable();
  if (HealPoints_CalcFrame:IsVisible()) then
    HealPointsCalculator:updateStats();
    HealPointsCalculatorUI:updateSelectedSpellStats();
  end
  HealPointsCalculator:updateHealPoints();
  HealPointsConfigUI:spellsChanged();
end

function HealPoints:TalentsChanged(delta) -- CHARACTER_POINTS_CHANGED
  lastLink = nil;
  HealPointsUtil:talentsChanged();
  HealPointsCalculator:talentsChanged();
  HealPointsCalculatorUI:updateSpellLists();
  HealPointsCalculatorUI:updateSpellTable();
  if (HealPoints_CalcFrame:IsVisible()) then
    HealPointsCalculator:updateStats();
    HealPointsCalculatorUI:updateSelectedSpellStats();
  end
  HealPointsCalculator:updateHealPoints();
end

function HealPoints:GearChanged()
  lastLink = nil;
  if (HealPoints_CalcFrame:IsVisible()) then
    HealPointsCalculatorUI:updateSpellTable();
    HealPointsCalculator:updateStats();
    HealPointsCalculatorUI:updateSelectedSpellStats();
  end
  if (HealPoints_CalcFrame:IsVisible() or CharacterFrame:IsVisible()) then
    HealPointsCalculator:updateHealPoints();
  end
end

function HealPoints:DrawTooltip(frame, reference)
	local function addLine(text, d1, d2)
		if (d1 == d2) then
			frame:AddDoubleLine(text, HealPointsUtil:colorValue(d1),
			NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
			NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		else
			frame:AddDoubleLine(text, HealPointsUtil:colorValue(d1).." / "..HealPointsUtil:colorValue(d2), 		
			NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
			NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);			
		end		
	end
	
	if (HealPoints:IsActive() and HealPoints.db.char.tooltips ~= 0 and reference ~= nil and string.find(reference, "item", 0, true) ~= nil) then
		if (reference ~= lastLink) then
			diff1, diff2, diff3, diff4 = HealPointsCalculator:computeHealpointsDiff(reference)
			lastLink = reference;
		end
		if (diff3 ~= nil and diff3 ~= 0 and diff4 ~= nil and diff4 ~= 0) then
			addLine("HealPoints difference, top slot:", diff1, diff2);
			addLine("HealPoints difference, bottom slot:", diff3, diff4);
			frame:Show();
		elseif (diff1 ~= 0 or diff2 ~= 0) then
			addLine("HealPoints difference:", diff1, diff2);
			frame:Show();
		end;
	end;
end

function HealPoints:clearLinkCache()
  lastLink = nil;
end

function HealPoints:listScannedBonuses()
  local _, className = UnitClass("player");
  self:Print("Special set/equip bonuses detected:");
  self:Print("===================================");
  if (className == "PALADIN") then
    self:Print("Reduces the casting time of your Holy Light spell by 0."..HealPointsBS:GetBonus('TIME_HL').." sec.");
    self:Print("Increases healing done by Flash of Light by up to "..HealPointsBS:GetBonus('AVG_ABS_FOL'));
    self:Print("Increases the healing from your Flash of Light ability by "..HealPointsBS:GetBonus('AVG_PC_FOL').."%");
    self:Print("Increases the benefit your Flash of Light and Holy Light spells receive from Blessing of Light by "..HealPointsBS:GetBonus('AVG_ABS_BOL'));
    self:Print("Increases healing done by Holy Light by up to "..HealPointsBS:GetBonus('AVG_ABS_HL'));
    self:Print("Reduces the mana cost of Holy Light by "..HealPointsBS:GetBonus('MANA_ABS_HL'));
  elseif (className == "PRIEST") then
    self:Print("-0."..HealPointsBS:GetBonus('TIME_FH').." sec to the casting time of your Flash Heal spell.");
    self:Print("Your Greater Heals now have a heal over time component equivalent to a rank "..HealPointsBS:GetBonus('GH_RENEW').." Renew.");
    self:Print("Reduces the mana cost of your Renew spell by "..HealPointsBS:GetBonus('MANA_PC_RENEW').."%");
    self:Print("Increases the duration of your Renew spell by "..HealPointsBS:GetBonus('DURATION_RENEW').." sec.");
    self:Print("Increases the healing from your Greater Heal ability by "..HealPointsBS:GetBonus('AVG_PC_GH').."%");
  elseif (className == "DRUID") then
    self:Print("Reduces the casting time of your Regrowth spell by 0."..HealPointsBS:GetBonus('TIME_REGR').." sec.");
    self:Print("Increases the duration of your Rejuvenation spell by "..HealPointsBS:GetBonus('DURATION_REJUV').." sec.");
    self:Print("Reduces the mana cost of your Healing Touch, Regrowth, Rejuvenation, and Tranquility spells by "..HealPointsBS:GetBonus('MANA_PC_DRUID').."%");
    self:Print("On Healing Touch critical hits, you regain "..HealPointsBS:GetBonus('MANA_REFUND_CRIT_HT').."% of the mana cost of the spell.");
    self:Print("Increases the duration of your Regrowth spell by "..HealPointsBS:GetBonus('DURATION_REGR').." sec.");
    self:Print("Increases the final amount healed by your Lifebloom spell by "..HealPointsBS:GetBonus('AVG_BURST_LIFEBL'));
    self:Print("Increases the healing from your Healing Touch ability by "..HealPointsBS:GetBonus('AVG_PC_HT').."%");
    self:Print("Increases the amount healed by Healing Touch by "..HealPointsBS:GetBonus('AVG_ABS_HT'));
    self:Print("Gain up to "..HealPointsBS:GetBonus('MANA_REFUND_HT').." mana each time you cast Healing Touch ");
    self:Print("Increases healing done by Rejuvenation by up to "..HealPointsBS:GetBonus('AVG_ABS_REJUV'));
    self:Print("Increases the periodic healing of your Lifebloom by up to "..HealPointsBS:GetBonus('AVG_HOT_LIFEBL'));
    self:Print("Reduces the mana cost of Regrowth by "..HealPointsBS:GetBonus('MANA_ABS_REGR'));
  elseif (className == "SHAMAN") then
    self:Print("After casting your Healing Wave or Lesser Healing Wave spell, gives you a 25% chance to gain Mana equal to "..HealPointsBS:GetBonus('MANA_REFUND_HWLHW').. "% of the base cost of the spell.");
    self:Print("Your Healing Wave will now jump to additional nearby targets. Each jump reduces the effectiveness of the heal by "..HealPointsBS:GetBonus('JUMP_HW').."%, and the spell will jump to up to two additional targets. (0% = no jumping)");
    self:Print("Increases the amount healed by Chain Heal to targets beyond the first by "..HealPointsBS:GetBonus('AVG_PC_JUMPS_CHAIN').."%");
    self:Print("Reduces the cost of your Lesser Healing Wave spell by "..HealPointsBS:GetBonus('MANA_PC_LHW').."%");
    self:Print("Reduces the casting time of your Healing Wave spell by 0."..HealPointsBS:GetBonus('TIME_HW').." sec.");
    self:Print("-0."..HealPointsBS:GetBonus('TIME_CHAIN').." seconds on the casting time of your Chain Heal spell.");
    self:Print("Your Chain Heal ability costs "..HealPointsBS:GetBonus('MANA_PC_CHAIN').."% less mana.");
    self:Print("Increases the amount healed by your Lesser Healing Wave ability by "..HealPointsBS:GetBonus('AVG_PC_LHW').."%");
    self:Print("Increases the base amount healed by Chain Heal by "..HealPointsBS:GetBonus('AVG_ABS_CHAIN'));
    self:Print("Increases healing done by Lesser Healing Wave by up to "..HealPointsBS:GetBonus('AVG_ABS_LHW'));
    self:Print("Reduces the mana cost of Heaing Wave by "..HealPointsBS:GetBonus('MANA_ABS_HW'));
    self:Print("Increases healing done by Healing Wave by up to "..HealPointsBS:GetBonus('AVG_ABS_HW'));
  end
end
